<?php 

/* 
 * functions.php
 * A collection of functions used throughout DaveDAP.
 */

require 'config.php';

/*
 * Returns the name of the image appropriate for the given $dn.
 * The image is found in the images directory
 */
function get_image_name( $dn )
{
	$dn = ldap_explode_dn( $dn, 0 );
	$rdn = $dn[0];
	$type = explode( '=', $rdn );
	$type = $type[0];

	switch( $type )
	{
		case 'dc': return 'dc.png';
		case 'ou': return 'ou.png';
		case 'uid': return 'uid.png';
		default: return 'object.png';
	}
}

/*
 * Given a server_id, returns whether or not we have enough information
 * to authenticate against the server. For example, if the user specifies
 * 'cookie' in the config for that server, it checks the $_COOKIE array to
 * see if the cookie username and password is set for the server.
 */
function have_auth_info( $server_id )
{
	global $servers;

	if( ! is_numeric( $server_id ) || ! isset( $servers[$server_id] ) )
		return false;

	$server = $servers[$server_id];

	if( $server['auth_type'] == 'form' )
	{
		global $_COOKIE;

		if( isset( $_COOKIE[ 'davedap_login_dn_' . $server_id ] ) &&
		    isset( $_COOKIE[ 'davedap_pass_' . $server_id ] ) )
			return true;
		else
			return false;
	}
	// whether or not the login_dn or pass is specified, we return 
	// true here. (if they are blank, we do an anonymous bind anyway)
	elseif( $server['auth_type'] == 'config' )
	{
		return true;
	}
	else
	{
		?>
		Error: You have an error in your config file. The only two allowed 
		values for 'auth_type' in the $servers section are 'config' and
		'form'. You entered '<?php echo htmlentities($server['auth_type']); ?>', which 
		is not allowed.
		<?php 
	}
}

/* 
 * Specify a $server_id (0,1,2...) based on the order it appears in config.php.
 * The first is 0, the second is 1, etc. You rarely will need to consult 
 * config.php since those values are usually generated dynamically in hrefs.
 */
function davedap_ldap_connect( $server_id )
{
	if( ! check_server_id( $server_id ) )
		return false;

	if( ! have_auth_info( $server_id ) )
		return false;

	global $servers;

	// cache the connection, so if we are called multiple
	// times, we don't have to reauthenticate with the LDAP server
	
	static $conns;
	if( $conns[$server_id] )
		return $conns[$server_id];



	$host = $servers[$server_id]['host'];
	$port = $servers[$server_id]['port'];
	if( ! $port ) $port = 389;

	$conn = @ldap_connect( $host, $port );
	
	// go with LDAP version 3 if possible (needed for renaming and Novell schema fetching)
	@ldap_set_option( $conn, LDAP_OPT_PROTOCOL_VERSION, 3 );

	if( ! $conn ) return false;

	// grab the auth info based on the auth_type for this server
	if( $servers[ $server_id ][ 'auth_type' ] == 'config' ) {
		$login_dn = $servers[$server_id]['login_dn'];
		$login_pass = $servers[$server_id]['login_pass'];
	} elseif( $servers[ $server_id ][ 'auth_type' ] == 'form' ) {
		global $_COOKIE;
		$login_dn = $_COOKIE['davedap_login_dn_' . $server_id ];
		$login_pass = $_COOKIE['davedap_pass_' . $server_id ];
	} else {
		die( "You have an error in your config file. auth_type of " . htmlentities( $servers[ $server_id ][ 'auth_type' ] ) .
			" is not valid." );
	}

	$res = @ldap_bind( $conn, $login_dn, $login_pass );

	if( ! $res ) return false;

	// store the cached connection resource
	$conns[$server_id] = $conn;

	return $conn;
}

/* 
 * Returns an array listing the DNs contained by the specified $dn
 */
function get_container_contents( $server_id, $dn )
{
	$con = davedap_ldap_connect( $server_id );
	if( ! $con ) return false;
	
	$search = @ldap_list( $con, $dn, 'objectClass=*', array( 'dn' ) );
	if( ! $search )
		return array();
	$search = ldap_get_entries( $con, $search );

	$return = array();

	for( $i=0; $i<$search['count']; $i++ )
	{
		$entry = $search[$i];
		$dn = $entry['dn'];
		$return[] = $dn;
	}

	return $return;
}

/* 
 * Builds the initial tree that is stored in the session variable 'tree'. 
 * Simply returns an array with an entry for each active server in
 * config.php
 */
function build_initial_tree()
{
	global $servers;
	$tree = array();
	foreach( $servers as $id => $server ) {
		if( $server['host'] == '' )
			continue;

		$dn = $server['base'];

		//$tree[$id][$dn] = array();
		$tree[$id] = array();
	}

	return $tree;
}

/* 
 * Returns the attribute/value pairs for the given $dn on the given
 * $server_id. If the attribute is single valued, it will return
 * a single value for that attribute. Otherwise, it will return an
 * array of values for that attribute. Here's a sample return value:
 *
 * Array
 * (
 *   [objectclass] => Array
 *       (
 *           [0] => organizationalRole
 *           [1] => krb5principal
 *           [2] => kerberosSecurityObject
 *       )
 *   [cn] => Manager
 *   [krbname] => phpldap@EXAMPLE.COM
 *   [dn] => cn=Manager,dc=example,dc=com
 * )
 */
function get_object_attrs( $server_id, $dn )
{
	$conn = davedap_ldap_connect( $server_id );
	if( ! $conn ) return false;

	$search = @ldap_read( $conn, $dn, 'objectClass=*' );

	if( ! $search )
		return false;

	$attrs = ldap_get_entries( $conn, $search );

	if( ! $attrs || $attrs['count'] == 0 ) 
		return false;

	$attrs = $attrs[0];
	$num_attrs = $attrs['count'];
	unset( $attrs['count'] );

	for( $i=0; $i<$num_attrs; $i++ )
		unset( $attrs[$i] );

	$return_array = array();
        foreach( $attrs as $attr => $vals ) {
		$count = $vals['count'];
		unset( $vals['count'] );
                if( $count == 1 )
                        $return_array[ $attr ] = $vals[0];
                else
                        $return_array[ $attr ] = $vals;
	}

	ksort( $return_array );

	return $return_array;
}

/* 
 * Returns true if the passed string $temp contains all printable 
 * ASCII characters. Otherwise (like if it contains binary data),
 * returns false.
 */
function is_printable_str($temp) {
	$len = strlen($temp);
	for ($i=0; $i<$len; $i++) {
		$ascii_val = ord( substr( $temp,$i,1 ) );
		if( $ascii_val < 32 || $ascii_val > 126 )
			return false;
	}

	return true;
}

/* 
 * Much like get_object_attrs(), but only returns the entry for
 * one attribute of an object. Again, if the attribute contains
 * multiple values, returns an array of values. Otherwise, it
 * returns the single attribute value.
 */
function get_object_attr( $server_id, $dn, $attr )
{
	$attr = strtolower( $attr );
	$attrs = get_object_attrs( $server_id, $dn );
	if( isset( $attrs[$attr] ) )
		return $attrs[$attr];
	else
		return false;
}

/* 
 * Returns true if $var is not white space only, and false otherwise.
 */
function not_white( $var )
{
	return trim($var) != "" ? true : false;
}

/* 
 * Returns an associative array of objectClasses for the specified 
 * $server_id. Each array entry's key is the name of the objectClass
 * in lower-case. 
 * The sub-entries consist of sub-arrays called 'must_attrs' and 
 * 'may_attrs', and sub-entries called 'oid', 'name' and 'description'.
 *
 * The bulk of this function came from the good code in the 
 * GPL'ed LDAP Explorer project. Thank you.
 */
function get_schema_objectclasses( $server_id )
{
	$ds = davedap_ldap_connect( $server_id );
	
	if( ! $ds )
		return false;
	
	// get all the objectClasses
	$result = @ldap_read($ds, 'cn=subschema', '(objectClass=*)',
				array( 'objectclasses' ), 0, 200, 0, LDAP_DEREF_ALWAYS );
	if( ! $result )
		$result = @ldap_read($ds, 'cn=schema', '(objectClass=*)',
				array( 'objectclasses' ), 0, 200, 0, LDAP_DEREF_ALWAYS );

	if( ! $result ) return false;
	if( $result ) $raw_oclasses = ldap_get_entries($ds,$result );
	
	// build the array of objectClasses
	$oclasses = array();
	for( $att=0; $att < count( $raw_oclasses[0]["objectclasses"] ); $att++ )
	{
		$class = $raw_oclasses[0]["objectclasses"][$att];

		preg_match( "/[\s]+NAME[\s'\(]+([a-zA-Z0-9\-_]+)[\s'\)]/" , $class, $name);
		preg_match( "/[\s]+([\d\.]+)[\s]+NAME/", $class, $oid );
		preg_match( "/[\s]+DESC[\s]+'([a-zA-Z0-9\-_ ]+)'/", $class, $description );
		preg_match( "/[\s]+SUP[\s]+([a-zA-Z0-9\-_]+)[\s]/", $class, $sup );

		$key = strtolower( trim( $name[1] ) );
		$oclass_name = trim( $name[1] );
		if( ! $key ) continue;

		$oclasses[$key] = array();
		$oclasses[$key]['oid'] = trim( $oid[1] );
		$oclasses[$key]['description'] = trim( $description[1] );
		$oclasses[$key]['sup'] = trim( $sup[1] );       
		
		unset( $name );
		unset( $syntax );
		unset( $desription );

		// get all the required attributes
		preg_match( "/MUST[\s\(]+([a-zA-Z0-9\s$]+)(MAY|\))/" , $class, $must_attrs );
		$must_attrs = str_replace( ' ', '', $must_attrs[1] );
		$oclasses[$key]['must_attrs'] = array_filter( explode( '$', $must_attrs ), "not_white" );

		// get all the optional attributes
		preg_match( "/MAY[\s\(]+([a-zA-Z0-9\s$]+)(MUST|\))/" , $class, $may_attrs );
		$may_attrs = str_replace( ' ', '', $may_attrs[1] );
		$oclasses[$key]['may_attrs'] = array_filter( array_merge( $oclasses[$key]['must_attrs'], explode( '$', $may_attrs) ), "not_white" );
		unset( $must_attrs );
		unset( $may_attrs );

		$oclasses[$key]['name'] = $oclass_name;
	}

	// go back and add any inherited MUST/MAY attrs to each objectClass
	foreach( $oclasses as $oclass => $attrs )
	{
		$new_must = $attrs['must_attrs'];
		$new_may = $attrs['may_attrs'];
		$sup_attr = $attrs['sup'];      

		while( $sup_attr && $sup_attr != "top" ) {
			$new_must = array_merge( $new_must, $oclasses[strtolower($sup_attr)]['must_attrs'] );
			$new_may = array_merge( $new_may, $oclasses[strtolower($sup_attr)]['may_attrs'] );
			$sup_attr = $oclasses[strtolower($sup_attr)]['sup'];
		}

		// TODO: flag these new attrs somehow to show that they were inherited... (maybe?)

		$oclasses[$oclass]['must_attrs'] = array_unique( $new_must );
		$oclasses[$oclass]['may_attrs'] = array_unique( $new_may );
	}

	ksort( $oclasses );

	return $oclasses;

}

/* 
 * Returns an associate array of the syntax OIDs that this LDAP server uses mapped to
 * their descriptions.
 */
function get_schema_syntaxes( $server_id )
{
	static $cache;

	if( isset( $cache[$server_id] ) )
		return $cache[$server_id];

	$ds = davedap_ldap_connect( $server_id );

	if( ! $ds )
		return false;

	// get all the attributeTypes
	$result = @ldap_read($ds, 'cn=subschema', '(objectClass=*)',
				array( 'ldapSyntaxes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );
	if( ! $result )
		$result = @ldap_read($ds, 'cn=schema', '(objectClass=*)',
				array( 'ldapSyntaxes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );

	if( $result )
		$raw = ldap_get_entries( $ds, $result );
	else
		return( array() );

	// build the array of attributes
	$syntaxes = array();
	for( $i=0; $i < $raw[0]['ldapsyntaxes']['count']; $i++ )
	{
		$syntax = $raw[0]['ldapsyntaxes'][$i];
		preg_match( "/[\s]+([\d\.]+)[\s]+/", $syntax, $oid);
		preg_match( "/[\s]+DESC[\s]+'([\)\(:?\.a-zA-Z0-9\-_ ]+)'/", $syntax, $description );

		$key = strtolower( trim( $oid[1] ) );
		if( ! $key ) continue;

		$syntaxes[$key] = array();
		$syntaxes[$key]['description'] = $description[1];
	}

	ksort( $syntaxes );

	$cache[$server_id] = $syntaxes;

	return $syntaxes;
}

/* 
 * Returns an associative array of attributes for the specified 
 * $server_id. Each array entry's key is the name of the attribute,
 * in lower-case.
 * The sub-entries are 'oid', 'syntax', 'equality', 'substr', 'name',
 * and 'single_value'.
 *
 * The bulk of this function came from the good code in the 
 * GPL'ed LDAP Explorer project. Thank you. It was extended
 * considerably for application here.
 */
function get_schema_attributes( $server_id )
{
	$ds = davedap_ldap_connect( $server_id );

	if( ! $ds )
		return false;

	// get all the attributeTypes
	$result = @ldap_read($ds, 'cn=subschema', '(objectClass=*)',
				array( 'attributeTypes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );
	if( ! $result )
		$result = @ldap_read($ds, 'cn=schema', '(objectClass=*)',
				array( 'attributeTypes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );

	if( $result )
		$raw_attrs = ldap_get_entries( $ds, $result );
	else
		$raw_attrs = array();

	$syntaxes = get_schema_syntaxes( $server_id );

	// build the array of attributes
	$attrs = array();
	for( $i=0; $i < $raw_attrs[0]['attributetypes']['count']; $i++ )
	{
		$attr = $raw_attrs[0]['attributetypes'][$i];
		
		preg_match( "/[\s]+NAME[\s'\(]+([a-zA-Z0-9\-_]+)[\s'\)]/" , $attr, $name);
		preg_match( "/[\s]+([\d\.]+)[\s]+NAME/", $attr, $oid );
		preg_match( "/[\s]+DESC[\s]+'([\)\(:?\.a-zA-Z0-9\-_ ]+)'/", $attr, $description );
		preg_match( "/[\s]+SYNTAX[\s]+([\d\.]+)/", $attr, $syntax);
		preg_match( "/[\s]+EQUALITY[\s]+([a-zA-Z]+)/", $attr, $equality);
		preg_match( "/[\s]+SUBSTR[\s]+([a-zA-Z]+)/", $attr, $substr);
		preg_match( "/[\s]+SUP[\s]+([a-zA-Z0-9\-_]+)/", $attr, $sup );

		if( preg_match( "/[\s]+SINGLE-VALUE[\s]+/", $attr, $single_value ) )
			$single_value = 'Yes';
		else
			$single_value = 'No';

		$key = strtolower( trim( $name[1] ) );
		$attr_name = trim( $name[1] );
		if( ! $key ) continue;

		$attrs[$key] = array();
		$attrs[$key]['oid'] = trim( $oid[1] );
		$attrs[$key]['description'] = trim( $description[1] );
		$attrs[$key]['syntax'] = trim( $syntax[1] );
		$attrs[$key]['type'] = $syntaxes[ trim($syntax[1]) ]['description'];
		$attrs[$key]['equality'] = trim( $equality[1] );
		$attrs[$key]['substr'] = trim( $substr[1] );
		$attrs[$key]['single_value'] = $single_value;
		$attrs[$key]['sup'] = trim( $sup[1] );
		$attrs[$key]['name'] = $attr_name;
	}

	// go back and add any inherited descriptions from parent attributes (ie, cn inherits name)
	foreach( $attrs as $attr => $desc )
	{
		$sup_attr = $desc['sup'];      
		while( $sup_attr ) {
			if( ! $attrs[ $sup_attr	]['sup'] )  {
				$attrs[ $attr ][ 'syntax' ]  = $attrs[ $sup_attr ]['syntax'];
				$attrs[ $attr ][ 'equality' ]  = $attrs[ $sup_attr ]['equality'];
				$attrs[ $attr ][ 'substr' ]  = $attrs[ $sup_attr ]['substr'];
				$attrs[ $attr ][ 'single_value' ]  = $attrs[ $sup_attr ]['single_value'];
				break;
			} else {
				$sup_attr = $attrs[ $sup_attr ]['sup'];
			}
		}
	}

	ksort( $attrs );

	return $attrs;
}

/* 
 * A wrapper function to save you from having to call get_schema_objectclasses()
 * and get_schema_attributes(). Returns an array with two indexes: 'oclasses'
 * and 'attributes', as defined by their respective functions above.
 */
function get_schema( $server_id )
{
	$ds = davedap_ldap_connect( $server_id );

	if( ! $ds )
		return false;

	$attrs = get_schema_attributes($server_id, $lower_case_all );
	$oclasses = get_schema_objectclasses($server_id, $lower_case_all );

	if( ! $oclasses )
		return false;

	$schema = array( 'attrs' => $attrs,
			 'oclasses' => $oclasses );
	return $schema;
}

/* 
 * A do-it-all ldap_search function. You can even specify the search scope. Other than
 * that, it's pretty much the same as the PHP ldap_search() call.
 */
function davedap_ldap_search( $server_id, $filter, $base_dn=null, $attrs=array(), $scope='sub', $sort_results=true )
{
	global $servers;

	if( ! isset($servers[$server_id]) || $servers[$server_id]['host'] == '' )
		return false;

	if( $base_dn == null )
		$base_dn = $servers[$server_id]['base'];
	
	$ds = davedap_ldap_connect( $server_id );
	if( ! $ds )
		return false;
	
	switch( $scope ) {
		case 'base':
			$search = @ldap_read( $ds, $base_dn, $filter, $attrs );
			break;
		case 'one':
			$search = @ldap_list( $ds, $base_dn, $filter, $attrs );
			break;
		case 'sub':
		default:
			$search = @ldap_search( $ds, $base_dn, $filter, $attrs );
			break;
	}

	if( ! $search )
		return array();

	$search = ldap_get_entries( $ds, $search );

	$return = array();
	foreach( $search as $id => $attrs ) {
		if( ! is_Array( $attrs ) )
			continue;
        	for( $i=0; $i<$attrs['count']; $i++ )
			unset( $attrs[$i] );
		$dn = $attrs['dn'];
		foreach( $attrs as $attr => $vals ) {
			$count = $vals['count'];
			unset( $vals['count'] );
			if( $count == 1 )
				$return[$dn][$attr] = $vals[0];
			else
				$return[$dn][$attr] = $vals;
		}
	}

	if( $sort_results ) ksort( $return );
	return $return;

}

/*
 * Transforms the user-configured lists into arrays and such.
 */
function process_config()
{
	global $search_result_attributes;
	$search_result_attributes = explode( ",", $search_result_attributes );
	array_walk( $search_result_attributes, "trim_it" );

	global $search_attributes_display;
	$search_attributes_display = explode( ",", $search_attributes_display );
	array_walk( $search_attributes_display, "trim_it" );

	global $search_attributes;
	$search_attributes= explode( ",", $search_attributes);
	array_walk( $search_attributes, "trim_it" );

	global $search_criteria_options;
	$search_criteria_options= explode( ",", $search_criteria_options);
	array_walk( $search_criteria_options, "trim_it" );
}

/*
 * Call-by-reference to trim a string
 */
function trim_it( &$str )
{
	$str = trim($str);
}

/*
 * Checks the server id for sanity. Ensures that the server is indeed in the configured list and active
 */
function check_server_id( $server_id )
{
	global $servers;
	if( ! is_numeric( $server_id ) || ! isset( $servers[$server_id] ) || $servers[$server_id]['host'] == '' )
		return false;
	else
		return true;
}

/*
 * Used to generate a random_sale for crypt-styoe passwords
 * --- added 20021125 by bayu irawan <bayuir@divnet.telkom.co.id> --- 
 */
function random_salt( $length )
{
        $possible = '0123456789'.
                        'abcdefghijklmnopqrstuvwxyz'.
                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $str = "";
        mt_srand((double)microtime() * 1000000);
        while( strlen( $str ) < $length )
        {
                $str .= substr( $possible, ( rand() % strlen( $possible ) ), 1 );
        }
        $str = "\$1\$".$str."\$";
        return $str;
}

/*
 * Goes through the user-configured server list and looks for an available server_id
 */
function get_avail_server_id()
{
	global $servers;

	for( $i=0; $i<count($servers); $i++ )
		if( check_server_id( $i ) &&  have_auth_info( $i ) )
			return $i;
	return false;
}


function get_rdn( $dn )
{
	$rdn = ldap_explode_dn( $dn, 0 );
	$rdn = $rdn[0];
	return $rdn;
}

function get_container( $dn )
{
	$rdn = ldap_explode_dn( $dn, 0 );
	$container = $rdn[ 1 ];
	for( $i=2; $i<count($rdn)-1; $i++ )
		$container .= ',' . $rdn[$i];
	return $container;
}

function davedap_verbose_error( $err_no )
{
	static $err_codes;
	if( count($err_codes) > 0 ) {
		echo "Using cache\n";
		return $err_codes[ $err_no ];
	} else {
		echo "Not using cache\n";
	}

	$err_codes_file = 'ldap_error_codes.txt';
	if( ! file_exists( $err_codes_file ) )
		return false;
	if( ! is_readable( $err_codes_file ) ) 
		return false;
	if( ! ($f = fopen( $err_codes_file, 'r' )) )
		return false;

	$contents = fread( $f, filesize( $err_codes_file ) );
	preg_match_all( "/0x[A-Fa-f0-9][A-Za-z0-9]\s+[0-9A-Za-z_]+\s+\"[^\"]*\"\n/", $contents, $entries );
	$err_codes = array();
	foreach( $entries[0] as $e )
	{
		preg_match( "/(0x[A-Za-z0-9][A-Za-z0-9])\s+([0-9A-Za-z_]+)\s+\"([^\"]*)\"/", $e, $entry );
		$hex_code = $entry[1];
		$title    = $entry[2];
		$desc     = $entry[3];
		$desc     = preg_replace( "/\s+/", " ", $desc );
		$err_codes[ $hex_code ] = array( 'title' => $title, 'desc' => $desc );
	}

	return $err_codes[ $err_no ];
}

?>
